
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>

#include <glib/glist.h>

#include "config.h"
#include "libconf.h"
#include "getstream.h"

static struct config_s	*config;
static struct adapter_s	*adapter;
static struct stream_s	*stream;
static struct channel_s	*channel;
static struct sap_s	*sap;

static int cf_adapter_start(char *t, char *v) {
	adapter=calloc(1, sizeof(struct adapter_s));
	adapter->no=strtol(v, NULL, 10);
	config->adapter=g_list_append(config->adapter, adapter);
	return 0;
}

static int cf_sap_scope(char *t, char *v) { sap->scope=v; return 0; }
static int cf_sap_sap_group(char *t, char *v) { sap->group=v; return 0; }
static int cf_sap_sap_port(char *t, char *v) { sap->port=strtol(v, NULL, 10); return 0; }
static int cf_sap_ttl(char *t, char *v) { sap->ttl=strtol(v, NULL, 10); return 0; }
static int cf_sap_announce_host(char *t, char *v) { sap->announcehost=v; return 0; }
static int cf_sap_announce_port(char *t, char *v) { sap->announceport=strtol(v, NULL, 10); return 0; }

static int cf_stream_remoteport(char *t, char *v) { stream->remoteport=strtol(v, NULL, 10); return 0; }
static int cf_stream_remoteaddr(char *t, char *v) { stream->remoteaddr=v; return 0; }
static int cf_stream_rtpport(char *t, char *v) { stream->rtpport=strtol(v, NULL, 10); return 0; }
static int cf_stream_rtcpport(char *t, char *v) { stream->rtcpport=strtol(v, NULL, 10); return 0; }
static int cf_stream_ttl(char *t, char *v) { stream->ttl=strtol(v, NULL, 10); return 0; }
static int cf_stream_url(char *t, char *v) { stream->url=v; return 0; }
static int cf_http_port(char *t, char *v) { config->http_port=strtol(v, NULL, 10); return 0; }

static int cf_sap_start(char *t, char *v) {
	sap=calloc(1, sizeof(struct sap_s));
	stream->sap=sap;
	sap->stream=stream;

	/* Default values */
	sap->ttl=15;
	return 0;
}

static int cf_channel_start(char *t, char *v) {
	channel=calloc(1, sizeof(struct channel_s));
	channel->id=strtol(v, NULL, 0);
	adapter->channel=g_list_append(adapter->channel, channel);
	return 0;
}

static int cf_stream_start(char *t, char *v, int stype) {
	stream=calloc(1, sizeof(struct stream_s));
	stream->type=stype;
	stream->ttl=15;
	channel->stream=g_list_append(channel->stream, stream);
	return 0;
}

static int cf_stream_udp_start(char *t, char *v) { return cf_stream_start(t, v, STYPE_UDP); }
static int cf_stream_rtp_start(char *t, char *v) { return cf_stream_start(t, v, STYPE_RTP); }
static int cf_stream_rtcp_start(char *t, char *v) { return cf_stream_start(t, v, STYPE_RTCP); }
static int cf_stream_http_start(char *t, char *v) { return cf_stream_start(t, v, STYPE_HTTP); }

struct lc_ventry conf_sap[] = {
	{ "scope", 0, 1, LCV_STRING, 0, NULL, cf_sap_scope },
	{ "sap-group", 0, 1, LCV_IPV4ADDR, 0, NULL, cf_sap_sap_group },
	{ "sap-port", 0, 1, LCV_NUM, 0, NULL, cf_sap_sap_port },
	{ "announce-host", 0, 1, LCV_STRING, 0, NULL, cf_sap_announce_host },
	{ "announce-port", 0, 1, LCV_NUM, 0, NULL, cf_sap_announce_port },
	{ "ttl", 0, 1, LCV_NUM, 0, NULL, cf_sap_ttl },
	{ NULL, 0, 0, 0, 0, NULL },
};

struct lc_ventry conf_stream_udp[] = {
	{ "remote-address", 1, 1, LCV_IPV4ADDR, 0, NULL, cf_stream_remoteaddr },
	{ "remote-port", 1, 1, LCV_NUM, 0, NULL, cf_stream_remoteport },
	{ "sap", 0, 1, LCV_NONE, 0, conf_sap, cf_sap_start },
	{ "ttl", 0, 1, LCV_NUM, 0, NULL, cf_stream_ttl },
	{ NULL, 0, 0, 0, 0, NULL, NULL },
};

struct lc_ventry conf_stream_rtp[] = {
	{ "remote-address", 1, 1, LCV_IPV4ADDR, 0, NULL, cf_stream_remoteaddr },
	{ "remote-port", 1, 1, LCV_NUM, 0, NULL, cf_stream_remoteport },
	{ "sap", 0, 1, LCV_NONE, 0, conf_sap, cf_sap_start },
	{ "ttl", 0, 1, LCV_NUM, 0, NULL, cf_stream_ttl },
	{ NULL, 0, 0, 0, 0, NULL },
};

struct lc_ventry conf_stream_rtcp[] = {
	{ "rtp-port", 1, 1, LCV_NUM, 0, NULL, cf_stream_rtpport },
	{ "rtcp-port", 1, 1, LCV_NUM, 0, NULL, cf_stream_rtcpport },
	{ "sap", 0, 1, LCV_NONE, 0, conf_sap, cf_sap_start },
	{ NULL, 0, 0, 0, 0, NULL },
};

struct lc_ventry conf_stream_http[] = {
	{ "url", 1, 1, LCV_STRING, 0, NULL, cf_stream_url },
	{ NULL, 0, 0, 0, 0, NULL },
};

static int cf_channel_name(char *t, char *v) { channel->name=v; return 0; }

static struct lc_ventry conf_channel[] = {
	{ "name", 1, 1, LCV_STRING, 0, NULL, cf_channel_name },
	{ "stream-udp", 0, 0, LCV_NONE, 0, conf_stream_udp, cf_stream_udp_start },
	{ "stream-rtp", 0, 0, LCV_NONE, 0, conf_stream_rtp, cf_stream_rtp_start },
	{ "stream-rtcp", 0, 0, LCV_NONE, 0, conf_stream_rtcp, cf_stream_rtcp_start },
	{ "stream-http", 0, 0, LCV_NONE, 0, conf_stream_http, cf_stream_http_start },
	{ NULL, 0, 0, 0, 0, NULL },
};

static int cf_dvbs_trans_freq(char *t, char *v) { adapter->dvbs.t_freq=strtol(v, NULL, 10); return 0; }
static int cf_dvbs_trans_pol(char *t, char *v) { adapter->dvbs.t_pol=v; return 0; }
static int cf_dvbs_trans_srate(char *t, char *v) { adapter->dvbs.t_srate=strtol(v, NULL, 10); return 0; }

static struct lc_ventry conf_dvbs_transponder[] = {
	{ "freq", 1, 1, LCV_NUM, 0, NULL, cf_dvbs_trans_freq },
	{ "pol", 1, 1, LCV_STRING, 0, NULL, cf_dvbs_trans_pol },
	{ "srate", 1, 1, LCV_NUM, 0, NULL, cf_dvbs_trans_srate },
	{ NULL, 0, 0, 0, 0, NULL },
};

static int cf_dvbs_lnb_lof1(char *t, char *v) { adapter->dvbs.lnb_lof1=strtol(v, NULL, 10); return 0; }
static int cf_dvbs_lnb_lof2(char *t, char *v) { adapter->dvbs.lnb_lof2=strtol(v, NULL, 10); return 0; }
static int cf_dvbs_lnb_slof(char *t, char *v) { adapter->dvbs.lnb_slof=strtol(v, NULL, 10); return 0; }

static struct lc_ventry conf_dvbs_lnb[] = {
	{ "lof1", 1, 1, LCV_NUM, 0, NULL, cf_dvbs_lnb_lof1 },
	{ "lof2", 1, 1, LCV_NUM, 0, NULL, cf_dvbs_lnb_lof2 },
	{ "slof", 1, 1, LCV_NUM, 0, NULL, cf_dvbs_lnb_slof },
	{ NULL, 0, 0, 0, 0, NULL },
};

static struct lc_ventry conf_dvbs[] = {
	{ "lnb", 1, 1, LCV_NONE, 0, conf_dvbs_lnb, NULL },
	{ "transponder", 1, 1, LCV_NONE, 0, conf_dvbs_transponder, NULL },
	{ NULL, 0, 0, 0, 0, NULL },
};

static int cf_dvbt_bandwidth(char *t, char *v) { adapter->dvbt.bandwidth=strtol(v, NULL, 10); return 0; }
static int cf_dvbt_freq(char *t, char *v) { adapter->dvbt.freq=strtol(v, NULL, 10); return 0; }
static int cf_dvbt_tmode(char *t, char *v) { adapter->dvbt.tmode=strtol(v, NULL, 10); return 0; }
static int cf_dvbt_modulation(char *t, char *v) { adapter->dvbt.modulation=strtol(v, NULL, 10); return 0; }
static int cf_dvbt_guard(char *t, char *v) { adapter->dvbt.guard=strtol(v, NULL, 10); return 0; }
static int cf_dvbt_hierarchy(char *t, char *v) { adapter->dvbt.hierarchy=strtol(v, NULL, 10); return 0; }

static struct lc_ventry conf_dvbt[] = {
	{ "bandwidth", 0, 1, LCV_NUM, 0, NULL, cf_dvbt_bandwidth },
	{ "frequency", 1, 1, LCV_NUM, 0, NULL, cf_dvbt_freq },
	{ "transmission-mode", 0, 1, LCV_NUM, 0, NULL, cf_dvbt_tmode },
	{ "modulation", 0, 1, LCV_NUM, 0, NULL, cf_dvbt_modulation },
	{ "guard-interval", 0, 1, LCV_NUM, 0, NULL, cf_dvbt_guard },
	{ "hierarchy", 0, 1, LCV_NUM, 0, NULL, cf_dvbt_hierarchy },
	{ NULL, 0, 0, 0, 0, NULL },
};

static int cf_dvbs(char *t, char *v) { adapter->type=AT_DVBS; return 0; }
static int cf_dvbt(char *t, char *v) { adapter->type=AT_DVBT; return 0; }

static struct lc_ventry conf_adapter[] = {
	{ "channel", 0, 0, LCV_NUM, LCO_UNIQ, conf_channel, cf_channel_start },
	{ "dvb-s", 0, 1, LCV_NONE, 0, conf_dvbs, cf_dvbs },
	{ "dvb-t", 0, 1, LCV_NONE, 0, conf_dvbt, cf_dvbt },
	{ NULL, 0, 0, 0, 0, NULL },
};


static struct lc_ventry conf_http[] = {
	{ "port", 1, 1, LCV_NUM, 0, NULL, cf_http_port },
};

static struct lc_ventry conf_main[] = {
	{ "adapter", 1, 1, LCV_NUM, LCO_UNIQ, conf_adapter, cf_adapter_start },
	{ "http", 1, 1, LCV_NONE, 0, conf_http, NULL },
	{ NULL, 0, 0, 0, 0, NULL },
};

struct config_s *readconfig(char *filename) {
	int			cfd;
	struct stat		sb;
	char			*ctext;
	struct lc_centry	*c;

	/* Base for config storage */
	config=calloc(1, sizeof(struct config_s));

	cfd=open(filename, O_RDONLY);
	if (cfd<0)
		return NULL;

	if (fstat(cfd, &sb)) {
		close(cfd);
		return NULL;
	}

	ctext=mmap(NULL, sb.st_size, PROT_READ, MAP_SHARED, cfd, 0);

	c=libconf_parse(ctext, sb.st_size);

	munmap(config, sb.st_size);
	close(cfd);

	if (!c)
		return NULL;

	if (!libconf_validate(c, conf_main)) {
		libconf_free(c);
		return NULL;
	}

	return config;
}
