#ifndef GETSTREAM_H
#define GETSTREAM_H

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <glib/glist.h>

#include <event.h>

#define DEBUG

#ifdef DEBUG
#define dprintf		printf
#else
#define dprintf( a... )
#endif

#define	MAX_CTL_MSG_SIZE	1500

/*
 *
 *
 * getstream.c
 *
 *
 */

extern int debuglvl;

struct channel_s {
	/* Config elements */
	int			id;
	char			*name;
	GList			*stream;
	/* Demux */
	unsigned int		apid, vpid, opid, ppid;
	char			pid0cc;
};

enum {
	AT_DVBS,
	AT_DVBT
};

struct adapter_s {
	int				no;		/* Adapter Number */
	int				type;		/* Adapter Type - DVB-S/DVB-T */

	struct {					/* Tuning information DVB-S */
		unsigned long	lnb_lof1;
		unsigned long	lnb_lof2;
		unsigned long	lnb_slof;

		unsigned long	t_freq;
		char		*t_pol;
		unsigned long	t_srate;
	} dvbs;

	struct {					/* Tuning information DVB-T */
		unsigned long	freq;
		int		bandwidth;		/* 0 (Auto) 6 (6Mhz), 7 (7Mhz), 8 (8Mhz) */
		int		tmode;			/* 0 (Auto) 2 (2Khz), 8 (8Khz) */
		int		modulation;		/* 0, 16, 32, 64, 128, 256 QUAM */
		int		guard;			/* 0, 4, 8, 16, 32 1/x Guard */
		int		hierarchy;		/* 0, -1, 1, 2, 4 - 0 Auto, -1 None */
	} dvbt;

	GList				*channel;
	int				fefd;
};

#include "stream.h"

/*
 *
 *
 * fe.c
 *
 *
 */
int fe_tune(struct adapter_s	*adapter);

/*
 *
 *
 * demux.c
 *
 *
 *
 */
#define MAX_MCAST_PAYLOAD	(1500-40)
#define TS_PACKET_SIZE		188
#define DVR_BUFFER_SIZE		188*10
#define MAX_PID			0x1fff

enum {
	PID_NONE,
	PID_PMT,
	PID_VIDEO,
	PID_AUDIO,
	PID_OTHER
};

struct pidentry_s {
	unsigned char		type,
				cc;
	time_t			lastvalid;
	struct channel_s	*channel;
};

int demux_init(struct adapter_s *adapter);

/*
 *
 *
 * util.c
 *
 *
 */

int addr_is_mcast(char *addr);
void dump_hex(uint8_t *buf, int size);
void ts_packet_decode(uint8_t *ts);

#endif
